/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.settings

import android.annotation.SuppressLint
import android.app.AlertDialog
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentConsentBinding
import dk.nsp.mingraviditet.extensions.onBackPressed
import dk.nsp.mingraviditet.ui.useradmission.UserAdmissionActivity
import dk.nsp.mingraviditet.util.android.OpenExternalLinksInBrowserWebViewClient
import dk.nsp.mingraviditet.util.android.WebViewHelper

@AndroidEntryPoint
class ConsentFragment : Fragment() {

    private val targetBaseUrl = "file:///android_asset/"

    private val webViewViewModel by viewModels<SettingsPageViewModel>()
    private val consentViewModel by viewModels<ConsentViewModel>()

    lateinit var binding: FragmentConsentBinding

    @SuppressLint("ClickableViewAccessibility")
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View? {
        return FragmentConsentBinding.inflate(inflater, container, false).apply {
            toolbar.setNavigationOnClickListener { onBackPressed() }
            toolbar.setTitle(R.string.settings_consent_title)

            webView.webViewClient =
                OpenExternalLinksInBrowserWebViewClient(root.context, onPageFinished = ::onWebViewPageFinished)

            webView.isVerticalScrollBarEnabled = false
            webView.setOnTouchListener { _, event -> (event?.action == MotionEvent.ACTION_MOVE) }

            webViewViewModel.onContent.observe(viewLifecycleOwner) {
                val htmlToLoad = WebViewHelper.createHtmlWithStyleSheet(rawHtml = it, type = WebViewHelper.Type.Subpage, context = root.context)
                webView.loadDataWithBaseURL(
                    targetBaseUrl,
                    htmlToLoad,
                    "text/html",
                    "UTF-8",
                    "",
                )
            }

            removeConsent.setOnClickListener {
                AlertDialog.Builder(requireContext())
                    .setTitle(R.string.settings_consent_remove_title)
                    .setMessage(R.string.settings_consent_remove_description_v1)
                    .setPositiveButton(R.string.common_delete) { _, _ ->
                        consentViewModel.clearPregnancy {
                            activity?.finishAffinity()
                            startActivity(Intent(requireContext(), UserAdmissionActivity::class.java))
                        }
                    }
                    .setNegativeButton(R.string.common_regret) { _, _ -> }
                    .setCancelable(true)
                    .show()
            }

            binding = this
        }.root
    }

    private fun onWebViewPageFinished(url: String?) {
        if (url?.startsWith(targetBaseUrl) == true) {
            binding.loadingSpinner.root.isVisible = false
            binding.removeConsent.isVisible = true
        }
    }
}
