/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.more.settings

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.compose.ui.platform.ComposeView
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.extensions.onBackPressed
import dk.nsp.mingraviditet.gravidcompose.extensions.setGravidContent
import dk.nsp.mingraviditet.model.misc.SettingSubSectionUrl
import dk.nsp.mingraviditet.ui.main.more.settings.compose.SettingsRoute
import dk.nsp.mingraviditet.util.extensions.navigateSafe
import timber.log.Timber

@AndroidEntryPoint
class SettingsFragment : Fragment() {
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {
        return ComposeView(requireContext()).apply {
            setGravidContent {
                SettingsRoute(
                    navigateToSubPage = {
                        when (it) {
                            SettingSubSectionUrl.WhoIsBehindTheApp,
                            SettingSubSectionUrl.DataProtection,
                            -> navigateToSubSettingPage(it)

                            SettingSubSectionUrl.Consent -> {
                                findNavController().navigateSafe(
                                    SettingsFragmentDirections.actionSettingsFragmentToConsentFragment(it.path),
                                )
                            }

                            else -> Timber.w("Tried to navigate to $it, which is not supported")
                        }
                    },
                    onBackPressed = { onBackPressed() },
                )
            }
        }.rootView
    }

    private fun navigateToSubSettingPage(subSection: SettingSubSectionUrl) {
        val title = when (subSection) {
            SettingSubSectionUrl.WhoIsBehindTheApp -> R.string.settings_who_is_behind_the_app_title
            SettingSubSectionUrl.DataProtection -> R.string.settings_data_protection_title
            else -> R.string.common_missing_data
        }

        findNavController().navigateSafe(
            SettingsFragmentDirections.actionSettingsFragmentToWebViewFragment(
                subSection.path,
                title,
            ),
        )
    }
}
