/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.datechooser

import android.os.Bundle
import android.os.Parcelable
import android.view.View
import androidx.fragment.app.activityViewModels
import androidx.navigation.fragment.navArgs
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.ui.widgets.BottomSheetViewModel
import dk.nsp.mingraviditet.ui.widgets.DefaultBottomSheetDialog
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled
import kotlinx.parcelize.Parcelize

@Parcelize
enum class DateChooserMethod : Parcelable {
    ExpectedDateOfDelivery,
    CalculateExpectedDateOfDelivery,
    DateOfBirth,
}

@AndroidEntryPoint
class DateChooserBottomSheet : DefaultBottomSheetDialog() {
    override val viewModel: BottomSheetViewModel by activityViewModels()

    override val forceFullscreen: Boolean = true

    private val args by navArgs<DateChooserBottomSheetArgs>()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        childFragmentManager.beginTransaction().apply {
            replace(
                binding.scrollView.id,
                DateChooserFragment().apply {
                    val bundle = Bundle()

                    // Get args key dynamically
                    args.toBundle().keySet().firstOrNull()?.let {
                        bundle.putString(DateChooserFragment.dateChooserMethodArgKey, it)
                    }

                    // Add actual bundle
                    bundle.putAll(args.toBundle())
                    arguments = bundle
                },
            )
        }.commitNow()

        viewModel.onClose.observeIfNotHandled(viewLifecycleOwner) {
            dismissAllowingStateLoss()
        }
    }
}
