/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
// Top-level build file where you can add configuration options common to all sub-projects/modules.
import com.github.benmanes.gradle.versions.updates.DependencyUpdatesTask


plugins {
    id("base") // Needed to add functionality to default "clean" task
    alias(libs.plugins.android.application) apply false
    alias(libs.plugins.android.library) apply false
    alias(libs.plugins.kotlin.jvm) apply false
    alias(libs.plugins.kotlin.serialization) apply false
    alias(libs.plugins.android.junit5) apply false
    alias(libs.plugins.hilt) apply false
    alias(libs.plugins.ksp) apply false

    alias(libs.plugins.gradle.versions)
    alias(libs.plugins.gradle.versions.update)
}

fun isNonStable(version: String): Boolean {
//    val stableKeyword = listOf("RELEASE", "FINAL", "GA").any { version.toUpperCase().contains(it) }
    val unstableKeyword = listOf("ALPHA", "BETA", "RC").any { version.uppercase().contains(it) }
    val regex = "^[0-9,.v-]+(-r)?$".toRegex()
    return unstableKeyword || regex.matches(version).not()
}

tasks.withType<DependencyUpdatesTask> {
    rejectVersionIf {
        isNonStable(candidate.version)
    }

    checkConstraints = true
    checkBuildEnvironmentConstraints = true

    outputFormatter = "json"
    outputDir = "build/dependencyUpdates"
    reportfileName = "report"
}

versionCatalogUpdate {
    keep {
        // keep versions without any library or plugin reference
        keepUnusedVersions.set(true)
        // keep all libraries that aren't used in the project
        keepUnusedLibraries.set(true)
        // keep all plugins that aren't used in the project
        keepUnusedPlugins.set(true)
    }
}

buildscript {
    extra.apply {
        // This is required as of Android Studio 4.1, this is required for modules/libraries since VERSION_CODE and VERSION_NAME is no longer included in the generated build config
        // See: https://issuetracker.google.com/issues/158695880
        // Use this field to set the major/minor/patch numbers for the whole app.
        set("versionName", "1.1.0")
        // Use this field to set the VersionCode for the whole app, this should not updated manually, but via `fastlane bumpReleaseVersion`
        set("versionCode", 1054)
    }

    val libs = extensions.getByType<VersionCatalogsExtension>().named("libs")
    dependencies {
        classpath(libs.findLibrary("android-navigation-gradlePlugin").get())
    }
}

tasks {
    clean {
        dependsOn(
            ":cleanQaReports",
            ":cleanBuildLogic",
        )
    }

    create<Delete>("cleanQaReports") {
        delete("${rootProject.rootDir}/qa-reports")
    }

    create<Delete>("cleanBuildLogic") {
        delete("${rootProject.rootDir}/build-logic/convention/build")
    }

    create("runAllUnitTests") {
        dependsOn(
            "app:testProjectEnvDebugUnitTest",
            "api:testProjectEnvDebugUnitTest",
            "repo:testProjectEnvDebugUnitTest",
            "shared:testProjectEnvDebugUnitTest",
        )
    }

    create("generateReports") {
        dependsOn(
            "app:jacocoProjectEnvReleaseReport",
            "api:jacocoProjectEnvReleaseReport",
            "repo:jacocoProjectEnvReleaseReport",
            "shared:jacocoProjectEnvReleaseReport",
        )
    }
}
